@@ -1,11 +1,13 @@
-from time import perf_counter
-import asyncio_dgram
 import socket
 import struct
+from time import perf_counter
+from typing import Optional
+
+import asyncio_dgram
 
 
 class BedrockServerStatus:
-    request_status_data = b'\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff\x00\xfe\xfe\xfe\xfe\xfd\xfd\xfd\xfd\x124Vx'
+    request_status_data = b"\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff\x00\xfe\xfe\xfe\xfe\xfd\xfd\xfd\xfd\x124Vx"
 
     def __init__(self, host, port=19132, timeout=3):
         self.host = host
@@ -13,27 +15,30 @@ class BedrockServerStatus:
         self.timeout = timeout
 
     @staticmethod
-    def parse_response(data: bytes, latency: int):
+    def parse_response(data: bytes, latency: float):
         data = data[1:]
-        name_length = struct.unpack('>H', data[32:34])[0]
-        data = data[34:34+name_length].decode().split(';')
+        name_length = struct.unpack(">H", data[32:34])[0]
+        decoded_data = data[34 : 34 + name_length].decode().split(";")
 
+        map_: Optional[str]
+        gamemode: Optional[str]
         try:
-            map_ = data[7]
-            gamemode = data[8]
+            map_ = decoded_data[7]
+            gamemode = decoded_data[8]
         except BaseException:
             map_ = None
             gamemode = None
 
         return BedrockStatusResponse(
-            protocol=data[2],
-            brand=data[0],
+            protocol=decoded_data[2],
+            brand=decoded_data[0],
+            version=decoded_data[3],
             latency=latency,
-            players_online=data[4],
-            players_max=data[5],
-            motd=data[1],
+            players_online=decoded_data[4],
+            players_max=decoded_data[5],
+            motd=decoded_data[1],
             map_=map_,
-            gamemode=gamemode
+            gamemode=gamemode,
         )
 
     def read_status(self):
@@ -66,12 +71,24 @@ class BedrockServerStatus:
 
 class BedrockStatusResponse:
     class Version:
-        def __init__(self, protocol, brand):
+        def __init__(self, protocol, brand, version):
             self.protocol = protocol
             self.brand = brand
+            self.version = version
 
-    def __init__(self, protocol, brand, latency, players_online, players_max, motd, map_, gamemode):
-        self.version = self.Version(protocol, brand)
+    def __init__(
+        self,
+        protocol,
+        brand,
+        version,
+        latency,
+        players_online,
+        players_max,
+        motd,
+        map_,
+        gamemode,
+    ):
+        self.version = self.Version(protocol, brand, version)
         self.latency = latency
         self.players_online = players_online
         self.players_max = players_max
