@@ -1,3 +1,5 @@
+import socket
+from typing import Any
 import click
 from json import dumps as json_dumps
 
@@ -6,7 +8,7 @@ from .. import MinecraftServer
 server = None
 
 
-@click.group(context_settings=dict(help_option_names=['-h', '--help']))
+@click.group(context_settings=dict(help_option_names=["-h", "--help"]))
 @click.argument("address")
 def cli(address):
     """
@@ -58,15 +60,14 @@ def status():
     """
     response = server.status()
     click.echo("version: v{} (protocol {})".format(response.version.name, response.version.protocol))
-    click.echo("description: \"{}\"".format(response.description))
+    click.echo('description: "{}"'.format(response.description))
     click.echo(
         "players: {}/{} {}".format(
             response.players.online,
             response.players.max,
-            [
-                "{} ({})".format(player.name, player.id)
-                for player in response.players.sample
-            ] if response.players.sample is not None else "No players online"
+            ["{} ({})".format(player.name, player.id) for player in response.players.sample]
+            if response.players.sample is not None
+            else "No players online",
         )
     )
 
@@ -77,28 +78,29 @@ def json():
     Prints server status and query in json. Supported by all Minecraft
     servers that are version 1.7 or higher.
     """
-    data = {'online': False}
+    data = {}
+    data["online"] = False
     # Build data with responses and quit on exception
     try:
         ping_res = server.ping()
-        data['online'] = True
-        data['ping'] = ping_res
+        data["online"] = True
+        data["ping"] = ping_res
 
         status_res = server.status(tries=1)
-        data['version'] = status_res.version.name
-        data['protocol'] = status_res.version.protocol
-        data['motd'] = status_res.description
-        data['player_count'] = status_res.players.online
-        data['player_max'] = status_res.players.max
-        data['players'] = []
+        data["version"] = status_res.version.name
+        data["protocol"] = status_res.version.protocol
+        data["motd"] = status_res.description
+        data["player_count"] = status_res.players.online
+        data["player_max"] = status_res.players.max
+        data["players"] = []
         if status_res.players.sample is not None:
-            data['players'] = [{'name': player.name, 'id': player.id} for player in status_res.players.sample]
+            data["players"] = [{"name": player.name, "id": player.id} for player in status_res.players.sample]
 
         query_res = server.query(tries=1)
-        data['host_ip'] = query_res.raw['hostip']
-        data['host_port'] = query_res.raw['hostport']
-        data['map'] = query_res.map
-        data['plugins'] = query_res.software.plugins
+        data["host_ip"] = query_res.raw["hostip"]
+        data["host_port"] = query_res.raw["hostport"]
+        data["map"] = query_res.map
+        data["plugins"] = query_res.software.plugins
     except:
         pass
     click.echo(json_dumps(data))
@@ -110,11 +112,19 @@ def query():
     Prints detailed server information. Must be enabled in
     servers' server.properties file.
     """
-    response = server.query()
-    click.echo("host: {}:{}".format(response.raw['hostip'], response.raw['hostport']))
+    try:
+        response = server.query()
+    except socket.timeout:
+        print(
+            """The server did not respond to the query protocol.
+              Please ensure that the server has enable-query turned on, and that the necessary port (same as server-port unless query-port is set) is open in any firewall(s).
+              See https://wiki.vg/Query for further information."""
+        )
+        raise click.Abort()
+    click.echo("host: {}:{}".format(response.raw["hostip"], response.raw["hostport"]))
     click.echo("software: v{} {}".format(response.software.version, response.software.brand))
     click.echo("plugins: {}".format(response.software.plugins))
-    click.echo("motd: \"{}\"".format(response.motd))
+    click.echo('motd: "{}"'.format(response.motd))
     click.echo(
         "players: {}/{} {}".format(
             response.players.online,
