@@ -1,10 +1,17 @@
-from mcstatus.pinger import ServerPinger, AsyncServerPinger
-from mcstatus.protocol.connection import TCPSocketConnection, UDPSocketConnection, TCPAsyncSocketConnection
-from mcstatus.querier import ServerQuerier
-from mcstatus.bedrock_status import BedrockServerStatus
+from mcstatus.pinger import PingResponse, ServerPinger, AsyncServerPinger
+from mcstatus.protocol.connection import (
+    TCPSocketConnection,
+    UDPSocketConnection,
+    TCPAsyncSocketConnection,
+    UDPAsyncSocketConnection,
+)
+from mcstatus.querier import QueryResponse, ServerQuerier, AsyncServerQuerier
+from mcstatus.bedrock_status import BedrockServerStatus, BedrockStatusResponse
 from mcstatus.scripts.address_tools import parse_address
 import dns.resolver
 
+__all__ = ["MinecraftServer", "MinecraftBedrockServer"]
+
 
 class MinecraftServer:
     """Base class for a Minecraft Java Edition server.
@@ -19,8 +26,8 @@ class MinecraftServer:
         self.host = host
         self.port = port
 
-    @staticmethod
-    def lookup(address: str):
+    @classmethod
+    def lookup(cls, address: str):
         """Parses the given address and checks DNS records for an SRV record that points to the Minecraft server.
 
         :param str address: The address of the Minecraft server, like `example.com:25565`.
@@ -32,7 +39,7 @@ class MinecraftServer:
         if port is None:
             port = 25565
             try:
-                answers = dns.resolver.query("_minecraft._tcp." + host, "SRV")
+                answers = dns.resolver.resolve("_minecraft._tcp." + host, "SRV")
                 if len(answers):
                     answer = answers[0]
                     host = str(answer.target).rstrip(".")
@@ -40,9 +47,9 @@ class MinecraftServer:
             except Exception:
                 pass
 
-        return MinecraftServer(host, port)
+        return cls(host, port)
 
-    def ping(self, tries: int = 3, **kwargs):
+    def ping(self, tries: int = 3, **kwargs) -> float:
         """Checks the latency between a Minecraft Java Edition server and the client (you).
 
         :param int tries: How many times to retry if it fails.
@@ -52,18 +59,18 @@ class MinecraftServer:
         """
 
         connection = TCPSocketConnection((self.host, self.port))
-        exception = None
-        for attempt in range(tries):
+        exception_to_raise_after_giving_up: Exception
+        for _ in range(tries):
             try:
                 pinger = ServerPinger(connection, host=self.host, port=self.port, **kwargs)
                 pinger.handshake()
                 return pinger.test_ping()
             except Exception as e:
-                exception = e
+                exception_to_raise_after_giving_up = e
         else:
-            raise exception
+            raise exception_to_raise_after_giving_up
 
-    async def async_ping(self, tries: int = 3, **kwargs):
+    async def async_ping(self, tries: int = 3, **kwargs) -> float:
         """Asynchronously checks the latency between a Minecraft Java Edition server and the client (you).
 
         :param int tries: How many times to retry if it fails.
@@ -72,19 +79,22 @@ class MinecraftServer:
         :rtype: float
         """
 
-        connection = await TCPAsyncSocketConnection((self.host, self.port))
-        exception = None
-        for attempt in range(tries):
+        connection = TCPAsyncSocketConnection()
+        await connection.connect((self.host, self.port))
+        exception_to_raise_after_giving_up: Exception
+        for _ in range(tries):
             try:
                 pinger = AsyncServerPinger(connection, host=self.host, port=self.port, **kwargs)
                 pinger.handshake()
-                return await pinger.test_ping()
+                ping = await pinger.test_ping()
+                connection.close()
+                return ping
             except Exception as e:
-                exception = e
+                exception_to_raise_after_giving_up = e
         else:
-            raise exception
+            raise exception_to_raise_after_giving_up
 
-    def status(self, tries: int = 3, **kwargs):
+    def status(self, tries: int = 3, **kwargs) -> PingResponse:
         """Checks the status of a Minecraft Java Edition server via the ping protocol.
 
         :param int tries: How many times to retry if it fails.
@@ -93,10 +103,9 @@ class MinecraftServer:
         :rtype: PingResponse
         """
 
-
         connection = TCPSocketConnection((self.host, self.port))
-        exception = None
-        for attempt in range(tries):
+        exception_to_raise_after_giving_up: Exception
+        for _ in range(tries):
             try:
                 pinger = ServerPinger(connection, host=self.host, port=self.port, **kwargs)
                 pinger.handshake()
@@ -104,11 +113,11 @@ class MinecraftServer:
                 result.latency = pinger.test_ping()
                 return result
             except Exception as e:
-                exception = e
+                exception_to_raise_after_giving_up = e
         else:
-            raise exception
+            raise exception_to_raise_after_giving_up
 
-    async def async_status(self, tries: int = 3, **kwargs):
+    async def async_status(self, tries: int = 3, **kwargs) -> PingResponse:
         """Asynchronously checks the status of a Minecraft Java Edition server via the ping protocol.
 
         :param int tries: How many times to retry if it fails.
@@ -119,8 +128,8 @@ class MinecraftServer:
 
         connection = TCPAsyncSocketConnection()
         await connection.connect((self.host, self.port))
-        exception = None
-        for attempt in range(tries):
+        exception_to_raise_after_giving_up: Exception
+        for _ in range(tries):
             try:
                 pinger = AsyncServerPinger(connection, host=self.host, port=self.port, **kwargs)
                 pinger.handshake()
@@ -128,40 +137,66 @@ class MinecraftServer:
                 result.latency = await pinger.test_ping()
                 return result
             except Exception as e:
-                exception = e
+                exception_to_raise_after_giving_up = e
         else:
-            raise exception
+            raise exception_to_raise_after_giving_up
 
-    def query(self, tries: int = 3):
+    def query(self, tries: int = 3) -> QueryResponse:
         """Checks the status of a Minecraft Java Edition server via the query protocol.
 
         :param int tries: How many times to retry if it fails.
         :return: Query status information in a `QueryResponse` instance.
         :rtype: QueryResponse
         """
-
-        exception = None
         host = self.host
         try:
-            answers = dns.resolver.query(host, "A")
+            answers = dns.resolver.resolve(host, "A")
             if len(answers):
                 answer = answers[0]
                 host = str(answer).rstrip(".")
         except Exception as e:
             pass
-        for attempt in range(tries):
+
+        exception_to_raise_after_giving_up: Exception
+        for _ in range(tries):
             try:
                 connection = UDPSocketConnection((host, self.port))
                 querier = ServerQuerier(connection)
                 querier.handshake()
                 return querier.read_query()
             except Exception as e:
-                exception = e
+                exception_to_raise_after_giving_up = e
         else:
-            raise exception
+            raise exception_to_raise_after_giving_up
 
-    async def async_query(self, tries: int = 3):
-        raise NotImplementedError # TODO: '-'
+    async def async_query(self, tries: int = 3) -> QueryResponse:
+        """Asynchronously checks the status of a Minecraft Java Edition server via the query protocol.
+
+        :param int tries: How many times to retry if it fails.
+        :return: Query status information in a `QueryResponse` instance.
+        :rtype: QueryResponse
+        """
+        host = self.host
+        try:
+            answers = dns.resolver.resolve(host, "A")
+            if len(answers):
+                answer = answers[0]
+                host = str(answer).rstrip(".")
+        except Exception as e:
+            pass
+
+        exception_to_raise_after_giving_up: BaseException
+        for _ in range(tries):
+            try:
+                connection = UDPAsyncSocketConnection()
+                await connection.connect((host, self.port))
+                querier = AsyncServerQuerier(connection)
+                await querier.handshake()
+                return await querier.read_query()
+            except Exception as e:
+                exception_to_raise_after_giving_up = e
+        else:
+            raise exception_to_raise_after_giving_up
 
 
 class MinecraftBedrockServer:
@@ -173,13 +208,9 @@ class MinecraftBedrockServer:
     :attr port:
     """
 
-    def __init__(self, host: str, port: int = 19132):
+    def __init__(self, host: str, port: int = None):
         self.host = host
-
-        if port is None:
-            self.port = 19132
-        else:
-            self.port = port
+        self.port = port or 19132
 
     @classmethod
     def lookup(cls, address: str):
@@ -189,10 +220,9 @@ class MinecraftBedrockServer:
         :return: A `MinecraftBedrockServer` instance.
         :rtype: MinecraftBedrockServer
         """
-
         return cls(*parse_address(address))
 
-    def status(self, tries: int = 3, **kwargs):
+    def status(self, tries: int = 3, **kwargs) -> BedrockStatusResponse:
         """Checks the status of a Minecraft Bedrock Edition server.
 
         :param int tries: How many times to retry if it fails.
@@ -200,7 +230,6 @@ class MinecraftBedrockServer:
         :return: Status information in a `BedrockStatusResponse` instance.
         :rtype: BedrockStatusResponse
         """
-
         exception = None
 
         for _ in range(tries):
@@ -215,7 +244,7 @@ class MinecraftBedrockServer:
 
         return resp
 
-    async def async_status(self, tries: int = 3, **kwargs):
+    async def async_status(self, tries: int = 3, **kwargs) -> BedrockStatusResponse:
         """Asynchronously checks the status of a Minecraft Bedrock Edition server.
 
         :param int tries: How many times to retry if it fails.
@@ -223,7 +252,6 @@ class MinecraftBedrockServer:
         :return: Status information in a `BedrockStatusResponse` instance.
         :rtype: BedrockStatusResponse
         """
-
         exception = None
 
         for _ in range(tries):
